/*
 * This file is part of broccoli.
 * Copyright (C) 2019 Chair of Applied Mechanics, Technical University of Munich
 * https://www.amm.mw.tum.de/
 */

#include "broccoli/io/filesystem.hpp"
#include "gtest/gtest.h"

using namespace broccoli;
using namespace io;

//! Check handling of files
TEST(filesystem, FileHandling)
{
    // Initialize helpers
    std::string testFile = "test_filesystem_FileHandling.txt";

    // Check, if file exists (could be true or false)
    if (filesystem::fileExists(testFile) == true) {
        // Remove file
        ASSERT_EQ(filesystem::removeFile(testFile), true);
    }

    // Check, if file exists (has to be gone now)
    ASSERT_EQ(filesystem::fileExists(testFile), false);

    // Create file
    std::ofstream newTestFile;
    newTestFile.open(testFile);
    newTestFile << "Test of file handling...\n";
    newTestFile.close();

    // Check, if file exists
    ASSERT_EQ(filesystem::fileExists(testFile), true);

    // Remove file
    ASSERT_EQ(filesystem::removeFile(testFile), true);

    // Check, if file exists (has to be gone now)
    ASSERT_EQ(filesystem::fileExists(testFile), false);
}

//! Check handling of empty directories
TEST(filesystem, DirectoryHandlingEmpty)
{
    // Initialize helpers
    std::string testDirectory = "test_filesystem_DirectoryHandlingEmpty";

    // Check, if directory exists (could be true or false)
    if (filesystem::directoryExists(testDirectory) == true) {
        // Remove directory
        ASSERT_EQ(filesystem::removeDirectory(testDirectory), true);
    }

    // Check, if directory exists (has to be gone now)
    ASSERT_EQ(filesystem::directoryExists(testDirectory), false);

    // Create directory
    ASSERT_EQ(filesystem::createDirectory(testDirectory), true);

    // Check, if directory exists
    ASSERT_EQ(filesystem::directoryExists(testDirectory), true);

    // Remove directory
    ASSERT_EQ(filesystem::removeDirectory(testDirectory), true);

    // Check, if directory exists (has to be gone now)
    ASSERT_EQ(filesystem::directoryExists(testDirectory), false);
}
